from random import choice, randrange

from django.core.management.base import BaseCommand
from django.utils import timezone
from faker import Faker

from project.testapp.models import Author, Book
from project.testapp.users.models import AppUser

fake = Faker()


class Command(BaseCommand):

    """Seed TestApp database"""

    def handle(self, *args, **options):
        self.stdout.write(self.style.HTTP_INFO('[TestApp] - Database seed'))

        AppUser.objects.create(email="user@example.com")

        Author.objects.all().delete()
        Book.objects.all().delete()

        authors = []

        names = [fake.unique.name() for i in range(2000)]
        titles = [fake.sentence(nb_words=randrange(30)) for i in range(5000)]

        for name in names:
            author_created = fake.date_time_between_dates(
                datetime_start='-10y',
                datetime_end='now',
                tzinfo=timezone.get_current_timezone(),
            )
            authors.append(
                Author.objects.create(
                    created=author_created,
                    name=name,
                    age=randrange(80),
                    gender=fake.random.choice((1, 2)),
                )
            )

        for title in titles:
            book_created = fake.date_time_between_dates(
                datetime_start='-10y',
                datetime_end='now',
                tzinfo=timezone.get_current_timezone(),
            )

            Book.objects.create(
                created=book_created, author=choice(authors), title=title
            )



from django.db import models


class Author(models.Model):
    created = models.DateTimeField('created')
    modified = models.DateTimeField('modified', auto_now=True)

    gender = models.IntegerField(choices=(('M', 1), ('F', 2)))
    name = models.CharField(max_length=32, unique=True)
    age = models.IntegerField()


class Book(models.Model):
    created = models.DateTimeField('created')
    modified = models.DateTimeField('modified', auto_now=True)

    title = models.CharField(max_length=128)
    author = models.ForeignKey(Author, related_name='books', on_delete=models.CASCADE)



from django.db import models

from django_insights.choices import BucketType


class BucketQuerySet(models.QuerySet):
    def timeseries(self):
        return self.filter(type=BucketType.TIMESERIES)

    def histograms(self):
        return self.filter(type=BucketType.HISTOGRAM)

    def scatterplots(self):
        return self.filter(type=BucketType.SCATTERPLOT)

    def barcharts(self):
        return self.filter(type=BucketType.BARCHART)


class BucketManager(models.Manager):
    def get_queryset(self):
        return BucketQuerySet(self.model, using=self._db)

    def timeseries(self):
        return self.get_queryset().timeseries()

    def histograms(self):
        return self.get_queryset().histograms()

    def scatterplots(self):
        return self.get_queryset().scatterplots()

    def barcharts(self):
        return self.get_queryset().barcharts()



from django.apps import AppConfig


class InsightAppConfig(AppConfig):
    name = 'django_insights'
    label = 'insights'
    verbose_name = 'Django Insights'

    default_auto_field = 'django.db.models.AutoField'



# Generated by Django 4.1.7 on 2023-03-27 19:26

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppUser',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                (
                    'is_superuser',
                    models.BooleanField(
                        default=False,
                        help_text='Designates that this user has all permissions without explicitly assigning them.',
                        verbose_name='superuser status',
                    ),
                ),
                ('created', models.DateTimeField(verbose_name='created')),
                (
                    'modified',
                    models.DateTimeField(auto_now=True, verbose_name='modified'),
                ),
                (
                    'email',
                    models.EmailField(
                        blank=True, default='', max_length=254, unique=True
                    ),
                ),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                (
                    'date_joined',
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    'groups',
                    models.ManyToManyField(
                        blank=True,
                        help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.',
                        related_name='user_set',
                        related_query_name='user',
                        to='auth.group',
                        verbose_name='groups',
                    ),
                ),
                (
                    'user_permissions',
                    models.ManyToManyField(
                        blank=True,
                        help_text='Specific permissions for this user.',
                        related_name='user_set',
                        related_query_name='user',
                        to='auth.permission',
                        verbose_name='user permissions',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf import settings
from django.conf.urls.static import static
from django.urls import include, path

urlpatterns = [
    path(
        '',
        include('django_insights.urls', namespace='insights'),
    ),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



from __future__ import annotations

from datetime import datetime
from typing import Any

from django.db.models import Avg, Case, Count, Value, When
from django.db.models.functions import Length, TruncMonth, TruncYear

from django_insights.metrics import metrics
from project.testapp.models import Author, Book

label = "Books"


@metrics.counter(
    question="How many authors are there in our store?",
    desc="Number of authors we sell books for in our store",
)
def count_authors() -> int:
    return Author.objects.count()


@metrics.counter(question="How many books are there?")
def count_books() -> int:
    return Book.objects.count()


@metrics.counter(question="Books with title longer than 20 chars?")
def count_books_title_gt_20() -> int:
    return (
        Book.objects.annotate(title_len=Length('title'))
        .filter(title_len__gt=20)
        .count()
    )


@metrics.counter(question="Books with title less than 10 chars?")
def count_books_title_lt_10() -> int:
    return (
        Book.objects.annotate(title_len=Length('title'))
        .filter(title_len__lt=10)
        .count()
    )


@metrics.counter(question="Authors with two or more books?")
def count_authors_with_two_or_more_books() -> int:
    return (
        Author.objects.prefetch_related('books')
        .annotate(total_books=Count('books'))
        .filter(total_books__gte=2)
        .count()
    )


@metrics.counter(question="Authors without books?")
def count_authors_without_books() -> int:
    return (
        Author.objects.prefetch_related('books')
        .annotate(total_books=Count('books'))
        .filter(total_books=0)
        .count()
    )


@metrics.counter(question="Authors with name longer than 20 chars?")
def count_authors_name_gt_20() -> int:
    return (
        Author.objects.annotate(name_len=Length('name')).filter(name_len__gt=20).count()
    )


@metrics.gauge(question="Average book(s) per author?")
def avg_books_per_author() -> int:
    avg_total_books = (
        Author.objects.prefetch_related('books')
        .annotate(total_books=Count('books'))
        .aggregate(Avg('total_books'))
        .get('total_books__avg')
    )

    return avg_total_books


@metrics.timeseries(
    question="Num of books created per month?",
    desc="How many books are added each month, since the opening of our store",
    xlabel="Month",
    xformat='%m',
    ylabel="Num of books",
)
def num_of_books_per_month() -> list[tuple[datetime, int]]:
    return (
        Book.objects.all()
        .annotate(month=TruncMonth('created'))
        .values('month')
        .filter(month__isnull=False)
        .annotate(total=Count('pk'))
        .values_list('month', 'total')
        .order_by('month')
    )


@metrics.timeseries(
    question="Num of books created per year?",
    xlabel="Year",
    xformat='%Y',
    ylabel="Num of books",
)
def num_of_books_per_year() -> list[tuple[datetime, int]]:
    return (
        Book.objects.all()
        .annotate(year=TruncYear('created'))
        .values('year')
        .filter(year__isnull=False)
        .annotate(total=Count('pk'))
        .values_list('year', 'total')
        .order_by('year')
    )


@metrics.scatterplot(
    question="Num of books by age of author?",
    xlabel="Age",
    ylabel="Num of books",
)
def author_age_vs_num_of_books() -> list[tuple[float, float, Any]]:
    return (
        Author.objects.values('age')
        .annotate(num_of_books=Count('books'), category=Value("author"))
        .values_list('num_of_books', 'age', 'category')
    )


@metrics.barchart(
    question="Num of books by gender of author?",
    xlabel="Gender",
    ylabel="Num of books",
)
def author_gender_vs_num_of_books() -> list[tuple[float, float, str]]:
    return (
        Author.objects.values('gender')
        .annotate(
            num_of_books=Count('books'),
            gender_category=Case(
                When(gender=1, then=Value('Male')),
                When(gender=2, then=Value('Female')),
            ),
        )
        .values_list('num_of_books', 'gender', 'gender_category')
    )



from django.urls import path

from django_insights import views

app_name = "insights"

urlpatterns = [
    path(
        'app/<uuid:app_uuid>/',
        views.InsightsAppView.as_view(),
        name="insight_app",
    ),
    path(
        '',
        views.InsightsDashboardView.as_view(),
        name="insight_dashboard",
    ),
    path(
        'charts/<int:bucket_id>/',
        views.InsightsChartView.as_view(),
        name="insight_chart",
    ),
]



from __future__ import annotations

import datetime
from dataclasses import dataclass
from typing import Any, NamedTuple, Optional


@dataclass
class CounterType:
    value: int


@dataclass
class GaugeType:
    value: float


class TimeSeriesAnswer(NamedTuple):
    timestamp: datetime.datetime
    xvalue: float


@dataclass
class TimeSeriesType:
    values: list[TimeSeriesAnswer]


class ScatterPlotAnswer(NamedTuple):
    xvalue: float
    yvalue: float
    category: Optional[Any]


@dataclass
class ScatterPlotType:
    values: list[ScatterPlotAnswer]


class BarChartAnswer(NamedTuple):
    xvalue: float
    yvalue: float
    category: Optional[Any]


@dataclass
class BarChartType:
    values: list[BarChartAnswer]
